#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2016 The Aerospace Corporation.
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import gr, gr_unittest
from gnuradio import blocks
from gnuradio.gr import tag_utils
from tagged_stream_frame_sync import tagged_stream_frame_sync
import time
import pmt

class qa_tagged_stream_frame_sync (gr_unittest.TestCase):

    def setUp (self):
        self.tb = gr.top_block ()

    def tearDown (self):
        self.tb = None

    def test_005 (self):
        """Test data alignment"""
            # test using ramp to test passed data
        tb = self.tb
        
        #offset, frame_size, gap_size, gap_offset, lock_after, unlock_after = (10, 25, 310, 400, 2, 6)
        #offset, frame_size, gap_size, gap_offset, lock_after, unlock_after = (22, 100, 310, 210, 4, 5)
        offset, frame_size, gap_size, gap_offset, lock_after, unlock_after = (22, 10, 610, 210, 4, 5)
        tag_key = 'TAG!'
        rate = 1e3
        num_tags = 10
        data_size = (num_tags + lock_after + unlock_after) * frame_size
        
        # data < 0 b/4 lock, > 0 after lock, should only see >= 0 at output
        data = [x - (lock_after - 1) * frame_size + 0j for x in range(data_size)] + [0,] * gap_size
        tagoffsets = [(frame_size * x) % data_size for x in range(num_tags)]
        tagoffsets = [x if x < gap_offset else x + gap_size for x in tagoffsets]
        taglist = [tag_utils.python_to_tag((x, pmt.intern(tag_key), pmt.PMT_T, pmt.PMT_NIL))
                   for x in tagoffsets]
        src = blocks.vector_source_c(data, repeat=True, tags=taglist)
        dly = blocks.delay(gr.sizeof_gr_complex, offset)                # offset samples rather than tag
        throttle = blocks.throttle(gr.sizeof_gr_complex, rate)

        fsync = tagged_stream_frame_sync(gr.sizeof_gr_complex, tag_key, frame_size, lock_after, unlock_after)
        snk = blocks.vector_sink_c()

        #print "--->> data_size = ", data_size, ", tag offsets @ ", sorted(tagoffsets)
        #print "--->> frame_size = ", frame_size, ", unlock_after = ", unlock_after, ", lock_after = ", lock_after
        tb.connect(src, throttle, dly, fsync, snk)
        try:
            tb.start()
            time.sleep(1)
            #print "--> wait for lock"
            fsync.locked.wait()
            #print "--> wait for unlock"
            while fsync.locked.is_set():
                time.sleep(0)
            fsync.locked.wait()
        except KeyboardInterrupt:
            pass
        finally:
            #print "STOP"
            tb.stop()
            tb.wait()

        import itertools as itt
        rxdata = snk.data()
        start_offset = (lock_after - 1) * frame_size
        unlock_size = (unlock_after + num_tags - lock_after) * frame_size
        checkdata = data[start_offset:start_offset + unlock_size]
        self.assertEquals(rxdata, tuple(checkdata[x % len(checkdata)] for x in range(len(rxdata))))

        #print "PASSED %d samples" % (len(rxdata)) \
        #      if not any(x - y for x, y in itt.izip(itt.cycle(
        #                data[start_offset:start_offset + unlock_size]), rxdata)) \
        #      else \
        #      "NOT PASSED %d" % (len(rxdata))


if __name__ == '__main__':
    gr_unittest.run(qa_tagged_stream_frame_sync, "qa_tagged_stream_frame_sync.xml")
